@@ -1,67 +1,81 @@
 /*
 
-Copyright (C) 1994-2013 John W. Eaton
+Copyright (C) 1994-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
-#if !defined (octave_fMatrix_h)
+#if ! defined (octave_fMatrix_h)
 #define octave_fMatrix_h 1
 
+#include "octave-config.h"
+
+#include "DET.h"
 #include "MArray.h"
 #include "MDiagArray2.h"
 #include "MatrixType.h"
-
+#include "fNDArray.h"
 #include "mx-defs.h"
 #include "mx-op-decl.h"
-#include "DET.h"
 
 class
 OCTAVE_API
-FloatMatrix : public MArray<float>
+FloatMatrix : public FloatNDArray
 {
 public:
 
   typedef FloatColumnVector column_vector_type;
   typedef FloatRowVector row_vector_type;
 
+  typedef FloatColumnVector real_column_vector_type;
+  typedef FloatRowVector real_row_vector_type;
+
+  typedef FloatMatrix real_matrix_type;
+  typedef FloatComplexMatrix complex_matrix_type;
+
+  typedef FloatDiagMatrix real_diag_matrix_type;
+  typedef FloatComplexDiagMatrix complex_diag_matrix_type;
+
+  typedef float real_elt_type;
+  typedef FloatComplex complex_elt_type;
+
   typedef void (*solve_singularity_handler) (float rcon);
 
-  FloatMatrix (void) : MArray<float> () { }
+  FloatMatrix (void) : FloatNDArray () { }
 
   FloatMatrix (octave_idx_type r, octave_idx_type c)
-    : MArray<float> (dim_vector (r, c)) { }
+    : FloatNDArray (dim_vector (r, c)) { }
 
   FloatMatrix (octave_idx_type r, octave_idx_type c, float val)
-    : MArray<float> (dim_vector (r, c), val) { }
+    : FloatNDArray (dim_vector (r, c), val) { }
 
-  FloatMatrix (const dim_vector& dv) : MArray<float> (dv.redim (2)) { }
+  FloatMatrix (const dim_vector& dv) : FloatNDArray (dv.redim (2)) { }
 
   FloatMatrix (const dim_vector& dv, float val)
-    : MArray<float> (dv.redim (2), val) { }
+    : FloatNDArray (dv.redim (2), val) { }
 
-  FloatMatrix (const FloatMatrix& a) : MArray<float> (a) { }
+  FloatMatrix (const FloatMatrix& a) : FloatNDArray (a) { }
 
-  template <class U>
-  FloatMatrix (const MArray<U>& a) : MArray<float> (a.as_matrix ()) { }
+  template <typename U>
+  FloatMatrix (const MArray<U>& a) : FloatNDArray (a.as_matrix ()) { }
 
-  template <class U>
-  FloatMatrix (const Array<U>& a) : MArray<float> (a.as_matrix ()) { }
+  template <typename U>
+  FloatMatrix (const Array<U>& a) : FloatNDArray (a.as_matrix ()) { }
 
   explicit FloatMatrix (const FloatRowVector& rv);
 
@@ -69,23 +83,24 @@ public:
 
   explicit FloatMatrix (const FloatDiagMatrix& a);
 
+  explicit FloatMatrix (const MDiagArray2<float>& a);
+
+  explicit FloatMatrix (const DiagArray2<float>& a);
+
   explicit FloatMatrix (const PermMatrix& a);
 
   explicit FloatMatrix (const boolMatrix& a);
 
   explicit FloatMatrix (const charMatrix& a);
 
-
-  FloatMatrix& operator = (const FloatMatrix& a)
-  {
-    MArray<float>::operator = (a);
-    return *this;
-  }
-
   bool operator == (const FloatMatrix& a) const;
   bool operator != (const FloatMatrix& a) const;
 
-  bool is_symmetric (void) const;
+  bool issymmetric (void) const;
+
+  OCTAVE_DEPRECATED (4.4, "use 'issymmetric' instead")
+  bool is_symmetric (void) const
+  { return issymmetric (); }
 
   // destructive insert/delete/reorder operations
 
@@ -117,6 +132,7 @@ public:
 
   friend class FloatComplexMatrix;
 
+  FloatMatrix hermitian (void) const { return MArray<float>::transpose (); }
   FloatMatrix transpose (void) const { return MArray<float>::transpose (); }
 
   // resize is the destructive equivalent for this one
@@ -139,22 +155,22 @@ public:
   }
 
 private:
-  FloatMatrix tinverse (MatrixType &mattype, octave_idx_type& info,
-                        float& rcon, int force, int calc_cond) const;
+  FloatMatrix tinverse (MatrixType& mattype, octave_idx_type& info,
+                        float& rcon, bool force, bool calc_cond) const;
 
-  FloatMatrix finverse (MatrixType &mattype, octave_idx_type& info,
-                        float& rcon, int force, int calc_cond) const;
+  FloatMatrix finverse (MatrixType& mattype, octave_idx_type& info,
+                        float& rcon, bool force, bool calc_cond) const;
 
 public:
   FloatMatrix inverse (void) const;
   FloatMatrix inverse (octave_idx_type& info) const;
-  FloatMatrix inverse (octave_idx_type& info, float& rcon, int force = 0,
-                       int calc_cond = 1) const;
+  FloatMatrix inverse (octave_idx_type& info, float& rcon, bool force = false,
+                       bool calc_cond = true) const;
 
-  FloatMatrix inverse (MatrixType &mattype) const;
-  FloatMatrix inverse (MatrixType &mattype, octave_idx_type& info) const;
-  FloatMatrix inverse (MatrixType &mattype, octave_idx_type& info, float& rcon,
-                       int force = 0, int calc_cond = 1) const;
+  FloatMatrix inverse (MatrixType& mattype) const;
+  FloatMatrix inverse (MatrixType& mattype, octave_idx_type& info) const;
+  FloatMatrix inverse (MatrixType& mattype, octave_idx_type& info, float& rcon,
+                       bool force = false, bool calc_cond = true) const;
 
   FloatMatrix pseudo_inverse (float tol = 0.0) const;
 
@@ -167,77 +183,78 @@ public:
   FloatDET determinant (void) const;
   FloatDET determinant (octave_idx_type& info) const;
   FloatDET determinant (octave_idx_type& info, float& rcon,
-                        int calc_cond = 1) const;
-  FloatDET determinant (MatrixType &mattype, octave_idx_type& info,
-                        float& rcon, int calc_cond = 1) const;
+                        bool calc_cond = true) const;
+  FloatDET determinant (MatrixType& mattype, octave_idx_type& info,
+                        float& rcon, bool calc_cond = true) const;
 
   float rcond (void) const;
-  float rcond (MatrixType &mattype) const;
+  float rcond (MatrixType& mattype) const;
 
 private:
   // Upper triangular matrix solvers
-  FloatMatrix utsolve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix utsolve (MatrixType& mattype, const FloatMatrix& b,
                        octave_idx_type& info,
                        float& rcon, solve_singularity_handler sing_handler,
                        bool calc_cond = false,
                        blas_trans_type transt = blas_no_trans) const;
 
   // Lower triangular matrix solvers
-  FloatMatrix ltsolve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix ltsolve (MatrixType& mattype, const FloatMatrix& b,
                        octave_idx_type& info,
                        float& rcon, solve_singularity_handler sing_handler,
                        bool calc_cond = false,
                        blas_trans_type transt = blas_no_trans) const;
 
   // Full matrix solvers (lu/cholesky)
-  FloatMatrix fsolve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix fsolve (MatrixType& mattype, const FloatMatrix& b,
                       octave_idx_type& info,
                       float& rcon, solve_singularity_handler sing_handler,
                       bool calc_cond = false) const;
 
 public:
   // Generic interface to solver with no probing of type
-  FloatMatrix solve (MatrixType &typ, const FloatMatrix& b) const;
-  FloatMatrix solve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix solve (MatrixType& mattype, const FloatMatrix& b) const;
+  FloatMatrix solve (MatrixType& mattype, const FloatMatrix& b,
                      octave_idx_type& info) const;
-  FloatMatrix solve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix solve (MatrixType& mattype, const FloatMatrix& b,
                      octave_idx_type& info, float& rcon) const;
-  FloatMatrix solve (MatrixType &typ, const FloatMatrix& b,
+  FloatMatrix solve (MatrixType& mattype, const FloatMatrix& b,
                      octave_idx_type& info, float& rcon,
                      solve_singularity_handler sing_handler,
                      bool singular_fallback = true,
                      blas_trans_type transt = blas_no_trans) const;
 
-  FloatComplexMatrix solve (MatrixType &typ, const FloatComplexMatrix& b) const;
-  FloatComplexMatrix solve (MatrixType &typ, const FloatComplexMatrix& b,
+  FloatComplexMatrix solve (MatrixType& mattype,
+                            const FloatComplexMatrix& b) const;
+  FloatComplexMatrix solve (MatrixType& mattype, const FloatComplexMatrix& b,
                             octave_idx_type& info) const;
-  FloatComplexMatrix solve (MatrixType &typ, const FloatComplexMatrix& b,
+  FloatComplexMatrix solve (MatrixType& mattype, const FloatComplexMatrix& b,
                             octave_idx_type& info, float& rcon) const;
-  FloatComplexMatrix solve (MatrixType &typ, const FloatComplexMatrix& b,
+  FloatComplexMatrix solve (MatrixType& mattype, const FloatComplexMatrix& b,
                             octave_idx_type& info, float& rcon,
                             solve_singularity_handler sing_handler,
                             bool singular_fallback = true,
                             blas_trans_type transt = blas_no_trans) const;
 
-  FloatColumnVector solve (MatrixType &typ, const FloatColumnVector& b) const;
-  FloatColumnVector solve (MatrixType &typ, const FloatColumnVector& b,
+  FloatColumnVector solve (MatrixType& mattype, const FloatColumnVector& b) const;
+  FloatColumnVector solve (MatrixType& mattype, const FloatColumnVector& b,
                            octave_idx_type& info) const;
-  FloatColumnVector solve (MatrixType &typ, const FloatColumnVector& b,
+  FloatColumnVector solve (MatrixType& mattype, const FloatColumnVector& b,
                            octave_idx_type& info, float& rcon) const;
-  FloatColumnVector solve (MatrixType &typ, const FloatColumnVector& b,
+  FloatColumnVector solve (MatrixType& mattype, const FloatColumnVector& b,
                            octave_idx_type& info, float& rcon,
                            solve_singularity_handler sing_handler,
                            blas_trans_type transt = blas_no_trans) const;
 
-  FloatComplexColumnVector solve (MatrixType &typ,
+  FloatComplexColumnVector solve (MatrixType& mattype,
                                   const FloatComplexColumnVector& b) const;
-  FloatComplexColumnVector solve (MatrixType &typ,
+  FloatComplexColumnVector solve (MatrixType& mattype,
                                   const FloatComplexColumnVector& b,
                                   octave_idx_type& info) const;
-  FloatComplexColumnVector solve (MatrixType &typ,
+  FloatComplexColumnVector solve (MatrixType& mattype,
                                   const FloatComplexColumnVector& b,
                                   octave_idx_type& info, float& rcon) const;
-  FloatComplexColumnVector solve (MatrixType &typ,
+  FloatComplexColumnVector solve (MatrixType& mattype,
                                   const FloatComplexColumnVector& b,
                                   octave_idx_type& info, float& rcon,
                                   solve_singularity_handler sing_handler,
@@ -300,7 +317,7 @@ public:
                               octave_idx_type& rank) const;
   FloatComplexMatrix lssolve (const FloatComplexMatrix& b,
                               octave_idx_type& info, octave_idx_type& rank,
-                              float &rcon) const;
+                              float& rcon) const;
 
   FloatColumnVector lssolve (const FloatColumnVector& b) const;
   FloatColumnVector lssolve (const FloatColumnVector& b,
@@ -323,24 +340,6 @@ public:
   FloatMatrix& operator += (const FloatDiagMatrix& a);
   FloatMatrix& operator -= (const FloatDiagMatrix& a);
 
-  // unary operations
-
-  boolMatrix operator ! (void) const;
-
-  // other operations
-
-  bool any_element_is_negative (bool = false) const;
-  bool any_element_is_positive (bool = false) const;
-  bool any_element_is_nan (void) const;
-  bool any_element_is_inf_or_nan (void) const;
-  bool any_element_not_one_or_zero (void) const;
-  bool all_elements_are_int_or_inf_or_nan (void) const;
-  bool all_integers (float& max_val, float& min_val) const;
-  bool too_large_for_float (void) const;
-
-  boolMatrix all (int dim = -1) const;
-  boolMatrix any (int dim = -1) const;
-
   FloatMatrix cumprod (int dim = -1) const;
   FloatMatrix cumsum (int dim = -1) const;
   FloatMatrix prod (int dim = -1) const;
@@ -408,7 +407,6 @@ extern OCTAVE_API FloatMatrix linspace (const FloatColumnVector& x1,
                                         const FloatColumnVector& x2,
                                         octave_idx_type n);
 
-
 MS_CMP_OP_DECLS (FloatMatrix, float, OCTAVE_API)
 MS_BOOL_OP_DECLS (FloatMatrix, float, OCTAVE_API)
 
@@ -420,7 +418,7 @@ MM_BOOL_OP_DECLS (FloatMatrix, FloatMatrix, OCTAVE_API)
 
 MARRAY_FORWARD_DEFS (MArray, FloatMatrix, float)
 
-template <class T>
+template <typename T>
 void read_int (std::istream& is, bool swap_bytes, T& val);
 
 #endif
